// Copyright 2016 Dietrich Epp.
// This file is part of the Moria Gamedev Repository.  This file is licensed
// under the terms of the MIT license.  For more information, see LICENSE.txt.
'use strict';
var util = require('./util');

var TriggerSpeed = 600, IdleSpeed = 50, IdlePause = 1000, TurnDistance = 4;

var ActorInfo = {
	'npc1': {eye: null},
	'npc2': {eye: 'eye1'},
	'npc3': {eye: 'eye3'},
	'npc4': {eye: 'eye1'},
};

function Actor(od, group, layers) {
	var x = 0, y = 0;
	this.id = od.id;
	this.pos = od.pos;
	var s = od.sprite;
	this.sprite = Game.make.sprite(0, 0, 'tile', s);
	this.sprite.anchor.set(0.5, 0.5);
	this.info = ActorInfo[s];
	if (this.info.eye) {
		this.eye = Game.make.sprite(0, 0, 'tile', this.info.eye);
		this.eye.anchor.set(0.5, 0.5);
		this.sprite.addChild(this.eye);
	} else {
		this.eye = null;
	}
	this.blinkTime = -1;
	this.blinkState = false;
	this.turnX = 0;
	this.turnLeft = true;
	var pos = this.getTarget(od.initial);
	this.sprite.x = pos[0][0];
	this.sprite.y = pos[0][1];
	this.setIdle(pos);
	group.addChild(this.sprite);
	layers.nonplayer.push(this.sprite);
	this.turnX = this.sprite.x + TurnDistance;
}

Actor.prototype.update = function() {
	var x = this.sprite.x;
	if (this.turnLeft) {
		if (x >= this.turnX) {
			this.turnLeft = false;
			this.turnX = x - TurnDistance;
			this.sprite.scale.set(-1, +1);
		} else {
			this.turnX = Math.min(this.turnX, x + TurnDistance);
		}
	} else {
		if (x <= this.turnX) {
			this.turnLeft = true;
			this.turnX = x + TurnDistance;
			this.sprite.scale.set(+1, +1);
		} else {
			this.turnX = Math.max(this.turnX, x - TurnDistance);
		}
	}
	if (this.eye && Game.time.now > this.blinkTime) {
		this.blinkState = !this.blinkState;
		if (this.blinkState) {
			this.eye.visible = false;
			this.blinkTime = Game.time.now + Game.rnd.integerInRange(100, 200);
		} else {
			this.eye.visible = true;
			this.blinkTime = Game.time.now + Game.rnd.integerInRange(3000, 7000);
		}
	}
};

Actor.prototype.getTarget = function(target) {
	var s = this.sprite;
	if (!target.length) {
		util.warn('Empty NPC position');
		return [[s.x, s.y]];
	}
	var pos = [];
	for (var i = 0; i < target.length; i++) {
		var p = this.pos[target[i]];
		if (!p) {
			util.warn('No such position: ' + this.id + '.' + target[i]);
			pos.push([s.x, s.y]);
		} else {
			pos.push([p[0] * 32 + 16, p[1] * 32 - 16]);
		}
	}
	return pos;
};

Actor.prototype.setIdle = function(pos) {
	if (pos.length > 1) {
		var x = pos[0][0], y = pos[0][1];
		pos.push(pos.shift());
		var tween = Game.add.tween(this.sprite);
		for (var i = 0; i < pos.length; i++) {
			var p = pos[i], nx = p[0], ny = p[1];
			var d = Math.max(Math.abs(nx - x), Math.abs(ny - y));
			tween.to(
				{x: nx, y: ny}, 1000 * d / IdleSpeed,
				undefined, undefined, IdlePause);
			x = nx;
			y = ny;
		}
		tween.loop(true).start();
	}
};

Actor.prototype.move = function(target, callback) {
	var pos = this.getTarget(target);
	var x = pos[0][0], y = pos[0][1];
	var distance = Math.max(
		Math.abs(this.sprite.x - x), Math.abs(this.sprite.y - y));
	var tween = Game.add.tween(this.sprite).to(
		{x: x, y: y},
		1000 * distance / TriggerSpeed,
		Phaser.Easing.Linear.None);
	tween.onComplete.addOnce(function() {
		this.setIdle(pos);
		callback();
	}, this);
	tween.start();
};

////////////////////////////////////////////////////////////////////////////////

function NPC() {
	this.actors = {};
}

NPC.prototype.create = function() {};

NPC.prototype.add = function(od, group, layers) {
	this.actors[od.id] = new Actor(od, group, layers);
};

NPC.prototype.update = function() {
	var a = this.actors, i;
	for (i in a) {
		a[i].update();
	}
};

NPC.prototype.move = function(actorId, target, callback) {
	var actor = this.actors[actorId];
	if (!actor) {
		util.warn('No such actor: ' + actor);
		callback();
		return;
	}
	actor.move(target, callback);
};

module.exports = {
	NPC: NPC,
};
