// Copyright 2016 Dietrich Epp.
// This file is part of the Moria Gamedev Repository.  This file is licensed
// under the terms of the MIT license.  For more information, see LICENSE.txt.
'use strict';
var util = require('./util');

var TeleportWidth = 24, TeleportHeight = 56;
var TeleportFrames = ['t1', 't2', 't3'];
var FrameTime = 50;

function Trigger() {
	this.triggers = [];
	this.teleportSprites = [];
	this.teleportTargets = {};
	this.frameTime = -1;
}

Trigger.prototype.create = function() {};

Trigger.prototype.add = function(od, group, layers) {
	var x, y, sp;
	switch (od.type) {
	case 'trigger':
		this.triggers.push({
			bounds: new Phaser.Rectangle(
				od.x * 32, od.y * 32, od.w * 32, od.h * 32),
			od: od,
			multiUse: false,
			action: triggerAction,
		});
		break;
	case 'teleport':
		x = od.x * 32 + 16;
		y = od.y * 32 - 16;
		this.triggers.push({
			bounds: new Phaser.Rectangle(
				x - TeleportWidth / 2, y - TeleportHeight / 2,
				TeleportWidth, TeleportHeight),
			od: od,
			multiUse: true,
			action: teleportAction,
		});
		sp = Game.add.sprite(x, y, 'tile', 't1', group);
		sp.anchor.set(0.5, 0.5);
		this.teleportSprites.push({x: x, y: y, sp: sp});
		layers.nonplayer.push(sp);
		break;
	case 'teleport_target':
		x = od.x * 32 + 16;
		y = od.y * 32 - 16;
		this.teleportTargets[od.id] = {x: x, y: y};
		break;
	}
};

Trigger.prototype.update = function(level) {
	var player = level.player.sprite;
	var x = player.x, y = player.y, a, i;
	for (a = this.triggers, i = 0; i < a.length; i++) {
		var trig = a[i];
		if (Phaser.Rectangle.contains(trig.bounds, x, y)) {
			if (!trig.multiUse) {
				a.splice(i, 1);
			}
			trig.action(level);
			return;
		}
	}
	if (this.teleportSprites.length) {
		var now = Game.time.now;
		if (now > this.frameTime) {
			this.frameTime = Math.max(now, this.frameTime + FrameTime);
			for (a = this.teleportSprites, i = 0; i < a.length; i++) {
				var sp = a[i];
				util.randomOffset(sp.sp, sp.x, sp.y);
				util.randomFrame(sp.sp, TeleportFrames);
			}
		}
	}
};

function triggerAction(level) {
	/*jshint validthis:true*/
	var od = this.od;
	var func1;
	if (od.exit) {
		func1 = function() {
			level.exit();
		};
	} else {
		func1 = function() {
			level.player.freeze(false);
		};
	}
	var func2 = func1;
	if (od.text) {
		func2 = function() {
			if (od.audio) {
				Game.sound.play('sfx/' + od.audio);
			}
			level.text.showText(od.text, func1);
		};
	}
	var func3 = func2;
	if (od.npc) {
		func3 = function() {
			level.npc.move(od.npc, od.pos, func2);
		};
	}
	level.player.freeze(true);
	func3();
}

function teleportAction(level) {
	/*jshint validthis:true*/
	var target = level.trigger.teleportTargets[this.od.id];
	if (!target) {
		util.warn('No teleport target');
		return;
	}
	var body = level.player.sprite.body.data;
	var phys = Game.physics.p2;
	phys.world.removeBody(body);
	body.position[0] = phys.pxmi(target.x);
	body.position[1] = phys.pxmi(target.y);
	phys.world.addBody(body);
	Game.sound.play('sfx/teleport');
}

module.exports = {
	Trigger: Trigger,
};
